﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

namespace $safeprojectname$
{
    public partial class MainPage : UserControl
    {

        private string[] _TaskList = new string[] {"Locate City", 
                                                   "Locate County"};

        public MainPage()
        {
            InitializeComponent();
            LayoutCanvas.SizeChanged += Canvas_SizeChanged;
            // Initialize mouse cursor handling
            MyMouseCursor.SetParent(this);
            MyMouseCursor.SetMap(MyMap);
            // Initialize navigation toolbar and associated dialogs
            MyNavToolbar.SetParent(this);
            MyNavToolbar.SetMap(MyMap);
            MyIdentifyDialog.SetMap(MyMap);
            MyIdentifyDialog.SetGraphicsLayer("IDGraphicsLayer");
            MyIdentifyDialog.SetTask("http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Demographics/ESRI_Census_USA/MapServer");
            MyMeasureDialog.SetParent(this);
            MyMeasureDialog.SetMap(MyMap);
            MyMeasureDialog.SetMapUnits(ScaleBarUnit.DecimalDegrees);
            MyMeasureDialog.SetGraphicsLayer("MeasureGraphicsLayer");
            // Initialize query tasks
            MyTaskMenu.SetParent(this);
            MyResultContainer.SetMap(MyMap);
            MyResultContainer.SetMapUnits(ScaleBarUnit.DecimalDegrees);
            MyResultContainer.SetGraphicsLayer("ResultsGraphicsLayer");
            MyResultContainer.SetIdentifyDialog(MyIdentifyDialog);
            MyCityLocator.SetResultContainer(MyResultContainer);
            MyCountyLocator.SetResultContainer(MyResultContainer);
        }

        // Mouse Cursor Handling
        public void SetCursor(MouseCursorType mcType)
        {
            MyMouseCursor.SetCursor(mcType);
        }

        // Tool Handling
        public void UnsetTool()
        {
            MyNavToolbar.UnsetTool();
            MyMeasureDialog.UnsetTool();
            MyMouseCursor.SetCursor(MouseCursorType.None);
        }

        public void Identify(MapPoint ptMapPoint)
        {
            MyIdentifyDialog.Identify(ptMapPoint);
        }

        public void ShowMeasureDialog()
        {
            MyMeasureDialog.Visibility = Visibility.Visible;
        }

        // Task Menu Handling
        public string[] GetTaskList()
        {
            return _TaskList;
        }

        public void OpenTask(string sTask)
        {
            switch (sTask)
            {
                case "Locate City":
                    MyCityLocator.Visibility = Visibility.Visible;
                    break;
                case "Locate County":
                    MyCountyLocator.Visibility = Visibility.Visible;
                    break;
            }
        }

        // Sizing Support
        private void Canvas_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            MyMap.Height = e.NewSize.Height;
            MyMap.Width = e.NewSize.Width;
            double dY0 = Canvas.GetTop(MyScaleBar);
            double dY = e.NewSize.Height - dY0 - 30;
            ScaleBarTransform.Y = dY;
        }

    }
}
