﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

namespace $safeprojectname$
{
    public partial class CityLocator : UserControl
    {

        private const string _StateQueryTaskURL = "http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Specialty/ESRI_StatesCitiesRivers_USA/MapServer/2";
        private const string _CityQueryTaskURL = "http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Specialty/ESRI_StatesCitiesRivers_USA/MapServer/0";
        private ResultContainer _ResultContainer;

        #region Public Methods
        public CityLocator()
        {
            InitializeComponent();
            btnOK.Click += btnOK_Click;
            MyDialogBase.SetParent(this);
            MyDialogBase.CloseEvent += MyDialogBase_Close;
            MyDialogBase.CollapseEvent += MyDialogBase_Collapse;
            MyDialogBase.ExpandEvent += MyDialogBase_Expand;
        }

        public void SetResultContainer(ResultContainer TheContainer)
        {

            _ResultContainer = TheContainer;

            // Set up query to populate state name combobox
            QueryTask qt = new QueryTask(_StateQueryTaskURL);
            qt.ExecuteCompleted += QueryTask_ExecuteCompleted;
            qt.Failed += QueryTask_Failed;
            Query q = new Query();
            q.OutFields.AddRange(new string[] { "state_name" });
            q.ReturnGeometry = false;
            q.Where = "1=1";
            qt.ExecuteAsync(q, "State");

        }
        #endregion

        #region Handlers
        private void MyDialogBase_Close(Object sender, EventArgs e)
        {
            this.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Collapse(Object sender, EventArgs e)
        {
            LocatorPanel.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Expand(Object sender, EventArgs e)
        {
            LocatorPanel.Visibility = Visibility.Visible;
        }

        private void btnOK_Click(Object sender, RoutedEventArgs e)
        {

            // Build whereclause
            string sWhereClause, sCityName, sStateName;
            sCityName = tbCity.Text;
            if (sCityName == "")
            {
                MessageBox.Show("No city name entered.");
                return;
            }
            if (sCityName.Contains("*") || sCityName.Contains("%"))
            {
                sCityName = sCityName.Replace("*", "%");
                sWhereClause = "city_name LIKE '" + sCityName + "'";
            }
            else
                sWhereClause = "city_name = '" + sCityName + "'";
            sStateName = cbState.SelectedItem.ToString();
            if (sStateName != "")
                sWhereClause = sWhereClause + " AND state_name = '" + sStateName + "'";

            // Execute query
            QueryTask qt = new QueryTask(_CityQueryTaskURL);
            qt.ExecuteCompleted += QueryTask_ExecuteCompleted;
            qt.Failed += QueryTask_Failed;
            Query q = new Query();
            q.Where = sWhereClause;
            q.ReturnGeometry = true;
            q.OutFields.Add("*");
            lblStatus.Text = "Querying...";
            qt.ExecuteAsync(q, "City");

        }

        private void QueryTask_ExecuteCompleted(Object sender, QueryEventArgs args)
        {
            FeatureSet fs = args.FeatureSet;
            string sQueryType = (string)args.UserState;
            if (sQueryType == "State")
                PopulateStates(fs);
            else
            {
                if (fs.Features.Count == 0)
                    MessageBox.Show("No features found");
                else
                    _ResultContainer.AddResults(fs, "Locate City");
                lblStatus.Text = "Done";
            }
        }

        private void QueryTask_Failed(Object sender, TaskFailedEventArgs args)
        {
            lblStatus.Text = "Error";
            MessageBox.Show("Query failed: " + args.Error.Message);
        }
        #endregion

        #region Private Methods
        private void PopulateStates(FeatureSet fs)
        {
            List<string> sl = new List<string>();
            foreach (Graphic g in fs.Features)
                sl.Add(g.Attributes["STATE_NAME"].ToString());
            sl.Sort();
            cbState.Items.Clear();
            cbState.Items.Add("");
            foreach (string s in sl)
                cbState.Items.Add(s);
            cbState.SelectedIndex = 0;
        }
        #endregion

    }
}
