﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

// **** Implementation of dialog with close, collapse, and expand behavior ****
// Parent must be arranged on a Canvas for this to work properly, 
// otherwise any resizing behavior will conflict with the
// layout mechanism.  (It can be done, but it's more difficult.)

namespace $safeprojectname$
{
    public partial class DialogBase : UserControl
    {

        private const double COLLAPSEHEIGHT = 28;

        private UserControl _Parent = null;
        private bool _bShowTitle = true;
        private bool _bShowCollapseButton = true;
        private bool _bShowCloseButton = true;
        private bool _bCollapsed = false;
        private double _dExpandedHeight;
        private bool _bResizable;

        public delegate void CloseEventHandler(Object sender, EventArgs e);
        public event CloseEventHandler CloseEvent;
        public delegate void CollapseEventHandler(Object sender, EventArgs e);
        public event CollapseEventHandler CollapseEvent;
        public delegate void ExpandEventHandler(Object sender, EventArgs e);
        public event ExpandEventHandler ExpandEvent;

        public DialogBase()
        {
            InitializeComponent();
            MyCloseButton.Click += MyCloseButton_Click;
            MyCollapseButton.Click += MyCollapseButton_Click;
        }

        public void SetParent(UserControl TheParent)
        {
            _Parent = TheParent;
            MyFloatingPanel.SetParent(TheParent);
        }

        public bool Floating
        {
            get { return MyFloatingPanel.Floating; }
            set { MyFloatingPanel.Floating = value; }
        }

        public bool Resizable
        {
            get { return MyFloatingPanel.Resizable; }
            set { MyFloatingPanel.Resizable = value; }
        }

        public bool ShowDialogTitle
        {
            get { return _bShowTitle; }
            set
            {
                _bShowTitle = value;
                if (_bShowTitle)
                    MyTitle.Visibility = Visibility.Visible;
                else
                    MyTitle.Visibility = Visibility.Collapsed;
            }
        }

        public string DialogTitleText
        {
            get { return MyTitle.Text; }
            set { MyTitle.Text = value; }
        }

        public bool ShowCollapseButton
        {
            get { return _bShowCollapseButton; }
            set
            {
                _bShowCollapseButton = value;
                if (_bShowCollapseButton)
                    MyCollapseButton.Visibility = Visibility.Visible;
                else
                    MyCollapseButton.Visibility = Visibility.Collapsed;
            }
        }

        public bool ShowCloseButton
        {
            get { return _bShowCloseButton; }
            set
            {
                _bShowCloseButton = value;
                if (_bShowCloseButton)
                    MyCloseButton.Visibility = Visibility.Visible;
                else
                    MyCloseButton.Visibility = Visibility.Collapsed;
            }
        }

        private void MyCollapseButton_Click(Object sender, RoutedEventArgs e)
        {
            UserControl ucTarget;
            if (_Parent == null)
                ucTarget = this;
            else
                ucTarget = _Parent;
            EventArgs args = EventArgs.Empty;
            Image img = (Image) MyCollapseButton.Content;
            if (_bCollapsed)
            {
                // Expand dialog
                if (_bResizable)
                    MyFloatingPanel.Resizable = true;
                ucTarget.Height = _dExpandedHeight;
                if (_Parent != null)
                    ExpandEvent(this, args);
                img.Style = (Style)Application.Current.Resources["CollapseButtonImage"];
                _bCollapsed = false;
            }
            else
            {
                // Collapse dialog
                _bResizable = MyFloatingPanel.Resizable;
                if (_bResizable)
                    MyFloatingPanel.Resizable = false;
                _dExpandedHeight = ucTarget.ActualHeight;
                if (_Parent != null)
                    CollapseEvent(this, args);
                ucTarget.Height = COLLAPSEHEIGHT;
                img.Style = (Style)Application.Current.Resources["ExpandButtonImage"];
                _bCollapsed = true;
            }
        }

        private void MyCloseButton_Click(Object sender, RoutedEventArgs e)
        {
            if (_Parent == null)
                this.Visibility = Visibility.Collapsed;
            else
            {
                EventArgs args = EventArgs.Empty;
                CloseEvent(this, args);
            }
        }
    
    }
}
