﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

// **** Implementation of floating, resizable placeholder ****
// Parent must be arranged on a Canvas for this to work properly, 
// otherwise any resizing behavior will conflict with the
// layout mechanism.  (It can be done, but it's more difficult.)

namespace $safeprojectname$
{
    public partial class FloatingPanel : UserControl
    {

        private UserControl _Parent = null;
        private bool _bFloating = true;
        private bool _bResizable = false;

        public FloatingPanel()
        {
            InitializeComponent();
            PanelBase.DragDelta += PanelBase_DragDelta;            
            PanelThumb.DragDelta += PanelGrip_DragDelta;
        }

        public void SetParent(UserControl TheParent)
        {
            _Parent = TheParent;
            _Parent.RenderTransform = FloatingPanelTransform;
            this.RenderTransform = null;
        }

        public bool Floating
        {
            get { return _bFloating; }
            set { _bFloating = value; }
        }

        public bool Resizable
        {
            get { return _bResizable; }
            set 
            { 
                _bResizable = value;
                if (_bResizable)
                {
                    Grip.Visibility = Visibility.Visible;
                    PanelThumb.Visibility = Visibility.Visible;
                }
                else
                {
                    Grip.Visibility = Visibility.Collapsed;
                    PanelThumb.Visibility = Visibility.Collapsed;
                }
            }
        }

        //**** Floating behavior

        private void PanelBase_DragDelta(Object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            if (!_bFloating)
                return;
            double dX = e.HorizontalChange;
            double dY = e.VerticalChange;
            FloatingPanelTransform.X += dX;
            FloatingPanelTransform.Y += dY;
        }

        //**** Resizing behavior

        private void PanelGrip_DragDelta(Object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            if (!_bResizable)
                return;
            UserControl ucTarget;
            if (_Parent == null)
                ucTarget = this;
            else
                ucTarget = _Parent;
            double dX = ucTarget.Width + e.HorizontalChange;
            double dY = ucTarget.Height + e.VerticalChange;
            if ((dX > ucTarget.MinWidth) && (dY > ucTarget.MinHeight))
            {
                ucTarget.Width = dX;
                ucTarget.Height = dY;
            }
        }

    }

}
