﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

//**** Adapted from ESRI's identify sample ****

namespace $safeprojectname$
{
    public partial class IdentifyDialog : UserControl
    {

        private class DataItem
        {
            private string privateTitle;
            public string Title
            {
                get { return privateTitle; }
                set { privateTitle = value; }
            }
            private IDictionary<string, Object> privateData;
            public IDictionary<string, Object> Data
            {
                get { return privateData; }
                set { privateData = value; }
            }
        }

        private Map _Map;
        private string _GraphicsLayer = "";
        private string _URL = "";
        private IdentifyTask _IdentifyTask;
        private List<DataItem> _dataItems = null;

        #region Public Methods
        public IdentifyDialog()
        {
            InitializeComponent();
            MyDialogBase.SetParent(this);
            MyDialogBase.CloseEvent += MyDialogBase_Close;
            MyDialogBase.CollapseEvent += MyDialogBase_Collapse;
            MyDialogBase.ExpandEvent += MyDialogBase_Expand;
            IdentifyComboBox.SelectionChanged += cb_SelectionChanged;
        }

        public void SetMap(Map TheMap)
        {
            _Map = TheMap;
        }

        public void SetGraphicsLayer(string sLayer)
        {
            _GraphicsLayer = sLayer;
        }

        public void SetTask(string TheURL)
        {
            _URL = TheURL;
            _IdentifyTask = new IdentifyTask(_URL);
            _IdentifyTask.ExecuteCompleted += IdentifyTask_ExecuteCompleted;
            _IdentifyTask.Failed += IdentifyTask_Failed;
        }

        public void Identify(MapPoint ptMapPoint)
        {
            if (_URL == "")
                return;
            IdentifyParameters identifyParams = new IdentifyParameters()
                {
                    Geometry = ptMapPoint,
                    MapExtent = _Map.Extent,
                    Width = (int)_Map.ActualWidth,
                    Height = (int)_Map.ActualHeight,
                    LayerOption = LayerOption.visible
                };
            _IdentifyTask.ExecuteAsync(identifyParams);
            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            graphicsLayer.ClearGraphics();
            Graphic graphic = new Graphic() { Geometry = ptMapPoint, Symbol = DefaultPictureSymbol };
            graphicsLayer.Graphics.Add(graphic);
        }

        public void IdentifyFeature(Graphic gFeat, string sTitle, bool bClear)
        {
            if (bClear)
            {
                IdentifyComboBox.Items.Clear();
                _dataItems = new List<DataItem>();
            }
            _dataItems.Add(new DataItem() { Title = sTitle, Data = gFeat.Attributes });
            if (this.Visibility == Visibility.Collapsed)
                this.Visibility = Visibility.Visible;
            IdentifyComboBox.Items.Add(sTitle);
            IdentifyComboBox.UpdateLayout();
            IdentifyComboBox.SelectedIndex = IdentifyComboBox.Items.Count - 1;
        }
        #endregion

        #region Handlers
        private void MyDialogBase_Close(Object sender, EventArgs e)
        {
            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            graphicsLayer.ClearGraphics();
            this.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Collapse(Object sender, EventArgs e)
        {
            IdentifyResultsPanel.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Expand(Object sender, EventArgs e)
        {
            IdentifyResultsPanel.Visibility = Visibility.Visible;
        }

        private void cb_SelectionChanged(Object sender, SelectionChangedEventArgs e)
        {
            int index = IdentifyComboBox.SelectedIndex;
            if (index > -1)
                IdentifyDetailsDataGrid.ItemsSource = _dataItems[index].Data;
        }

        private void IdentifyTask_ExecuteCompleted(Object sender, IdentifyEventArgs args)
        {
            IdentifyDetailsDataGrid.ItemsSource = null;
            if (this.Visibility == Visibility.Collapsed)
                this.Visibility = Visibility.Visible;
            if ((args.IdentifyResults != null) && (args.IdentifyResults.Count > 0))
            {
                IdentifyResultsPanel.Visibility = Visibility.Visible;
                ShowFeatures(args.IdentifyResults);
            }
            else
            {
                IdentifyComboBox.Items.Clear();
                IdentifyComboBox.UpdateLayout();
                IdentifyResultsPanel.Visibility = Visibility.Collapsed;
            }
        }

        private void IdentifyTask_Failed(Object sender, TaskFailedEventArgs e)
        {
            MessageBox.Show("Identify failed. Error: " + e.Error.Message);
        }
        #endregion

        #region Private Methods
        private void ShowFeatures(List<IdentifyResult> results)
        {
            _dataItems = new List<DataItem>();
            if ((results != null) && (results.Count > 0))
            {
                IdentifyComboBox.Items.Clear();
                foreach (IdentifyResult result in results)
                {
                    Graphic feature = result.Feature;
                    string title = result.Value.ToString() + " (" + result.LayerName + ")";
                    _dataItems.Add(new DataItem() { Title = title, Data = feature.Attributes });
                    IdentifyComboBox.Items.Add(title);
                }
                IdentifyComboBox.UpdateLayout();
                IdentifyComboBox.SelectedIndex = 0;
            }
        }
        #endregion

    }
}
