﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;

namespace $safeprojectname$
{

    public enum MouseCursorType
    {
        None = 0,
        ZoomIn = 1,
        ZoomOut = 2,
        Pan = 3,
        Identify = 4,
        Measure = 5
    }

    public partial class MouseCursor : UserControl
    {

        private MainPage _Parent;
        private Map _Map;
        private bool _bActive;
        private bool _bInMap;
        private Point _ptHotSpot;
        private Point _ptSize;

        public MouseCursor()
        {
            InitializeComponent();
        }

        public void SetParent(MainPage TheParent)
        {
            _Parent = TheParent;
            //_Parent.MouseEnter += _Parent_MouseEnter;
            _Parent.MouseMove += _Parent_MouseMove;
            //_Parent.MouseLeave += _Parent_MouseLeave;
        }

        public void SetMap(Map TheMap)
        {
            _Map = TheMap;
            _Map.MouseEnter += _Map_MouseEnter;
            _Map.MouseLeave += _Map_MouseLeave;
        }

        public void SetCursor(MouseCursorType mcType)
        {
            switch (mcType)
            {
                case MouseCursorType.None:
                    _bActive = false;
                    _Parent.Cursor = Cursors.Arrow;
                    this.Visibility = Visibility.Collapsed;
                    break;
                case MouseCursorType.ZoomIn:
                    _bActive = true;
                    CursorZoomIn.Visibility = Visibility.Visible;
                    SetHotspot(CursorZoomIn);
                    CursorZoomOut.Visibility = Visibility.Collapsed;
                    CursorPan.Visibility = Visibility.Collapsed;
                    CursorIdentify.Visibility = Visibility.Collapsed;
                    CursorMeasure.Visibility = Visibility.Collapsed;
                    break;
                case MouseCursorType.ZoomOut:
                    _bActive = true;
                    CursorZoomIn.Visibility = Visibility.Collapsed;
                    CursorZoomOut.Visibility = Visibility.Visible;
                    SetHotspot(CursorZoomOut);
                    CursorPan.Visibility = Visibility.Collapsed;
                    CursorIdentify.Visibility = Visibility.Collapsed;
                    CursorMeasure.Visibility = Visibility.Collapsed;
                   break;
                case MouseCursorType.Pan:
                    _bActive = true;
                    CursorZoomIn.Visibility = Visibility.Collapsed;
                    CursorZoomOut.Visibility = Visibility.Collapsed;
                    CursorPan.Visibility = Visibility.Visible;
                    SetHotspot(CursorPan);
                    CursorIdentify.Visibility = Visibility.Collapsed;
                    CursorMeasure.Visibility = Visibility.Collapsed;
                    break;
                case MouseCursorType.Identify:
                    _bActive = true;
                    CursorZoomIn.Visibility = Visibility.Collapsed;
                    CursorZoomOut.Visibility = Visibility.Collapsed;
                    CursorPan.Visibility = Visibility.Collapsed;
                    CursorIdentify.Visibility = Visibility.Visible;
                    SetHotspot(CursorIdentify);
                    CursorMeasure.Visibility = Visibility.Collapsed;
                    break;
                case MouseCursorType.Measure:
                    _bActive = true;
                    CursorZoomIn.Visibility = Visibility.Collapsed;
                    CursorZoomOut.Visibility = Visibility.Collapsed;
                    CursorPan.Visibility = Visibility.Collapsed;
                    CursorIdentify.Visibility = Visibility.Collapsed;
                    CursorMeasure.Visibility = Visibility.Visible;
                    SetHotspot(CursorMeasure);
                    break;
            }
        }

        //private void _Parent_MouseEnter(Object sender, MouseEventArgs args)
        //{
        //    if (!_bActive)
        //        return;
        //    _Parent.Cursor = Cursors.None;
        //    this.Visibility = Visibility.Visible;
        //    MoveTo(args);
        //}

        private void _Parent_MouseMove(Object sender, MouseEventArgs args)
        {
            if (!_bActive)
                return;
            if (!_bInMap)
                return;
            MoveTo(args);
        }

        //private void _Parent_MouseLeave(Object sender, MouseEventArgs args)
        //{
        //    if (!_bActive)
        //        return;
        //    this.Visibility = Visibility.Collapsed;
        //}

        private void _Map_MouseEnter(Object sender, MouseEventArgs args)
        {
            if (!_bActive)
                return;
            _bInMap = true;
            _Parent.Cursor = Cursors.None;
            this.Visibility = Visibility.Visible;
            MoveTo(args);
        }

        private void _Map_MouseLeave(Object sender, MouseEventArgs args)
        {
            if (!_bActive)
                return;
            _bInMap = false;
            _Parent.Cursor = Cursors.Arrow;
            this.Visibility = Visibility.Collapsed;
        }

        private void SetHotspot(Image img)
        {
            string sTag = img.Tag.ToString();
            string[] Tokens = sTag.Split(new char[] { ':' });
            string[] Point = Tokens[1].Split(new char[] { ',' });
            _ptHotSpot.X = Convert.ToInt32(Point[0]);
            _ptHotSpot.Y = Convert.ToInt32(Point[1]);
            _ptSize.X = img.Width;
            _ptSize.Y = img.Height;
        }

        private void MoveTo(MouseEventArgs args)
        {
            Point pt2 = args.GetPosition(this);
            GridTransform.X = pt2.X - _ptHotSpot.X - ((32 - _ptSize.X) / 2);
            GridTransform.Y = pt2.Y - _ptHotSpot.Y - ((32 - _ptSize.Y) / 2);
        }

    }

}
