﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

//**** Adapted from ESRI's toolbar sample ****

namespace $safeprojectname$
{
    public partial class NavToolbar : UserControl
    {

        private MainPage _Parent;
        private Map _Map;
        private Draw _ZoomDrawObject;

        private string _sToolMode = "";
        private List<Envelope> _listExtentHistory = new List<Envelope>();
        private int _iCurrentExtentIndex = 0;
        private bool _bNewExtent = true;
        private Button _btnCurrentTool = null;

        #region Public Methods
        public NavToolbar()
        {
            InitializeComponent();
            MyFloatingPanel.SetParent(this);
            btnZoomIn.Click += btnZoomIn_Click;
            btnZoomOut.Click += btnZoomOut_Click;
            btnPan.Click += btnPan_Click;
            btnZoomPrev.Click += btnZoomPrev_Click;
            btnZoomNext.Click += btnZoomNext_Click;
            btnZoomFull.Click += btnZoomFull_Click;
            btnIdentify.Click += btnIdentify_Click;
            btnMeasure.Click += btnMeasure_Click;
        }

        public void SetParent(MainPage TheParent)
        {
            // Assumes that MainPage supports SetCursor and UnsetTool
            _Parent = TheParent;
        }

        public void SetMap(Map TheMap)
        {
            _Map = TheMap;
            _Map.ExtentChanged += _Map_ExtentChanged;
            _Map.MouseClick += _Map_MouseClick;
            _ZoomDrawObject = new Draw(_Map) { FillSymbol = DefaultFillSymbol, DrawMode = DrawMode.Rectangle };
            _ZoomDrawObject.DrawComplete += _ZoomDrawObject_DrawComplete;
        }

        public void UnsetTool()
        {
            if (_btnCurrentTool == null)
                return;
            _btnCurrentTool.Background = (Brush) Application.Current.Resources["ButtonNormalBrush"];
            _ZoomDrawObject.IsEnabled = false;
            _btnCurrentTool = null;

        }
        #endregion

        #region Handlers
        private void _ZoomDrawObject_DrawComplete(Object sender, DrawEventArgs args)
        {
            if (_sToolMode == "zoomin")
                _Map.ZoomTo((Envelope) args.Geometry);
            else if (_sToolMode == "zoomout")
            {
                Envelope currentExtent = _Map.Extent;
                Envelope zoomBoxExtent = (Envelope) args.Geometry;
                MapPoint zoomBoxCenter = zoomBoxExtent.GetCenter();
                double whRatioCurrent = currentExtent.Width / currentExtent.Height;
                double whRatioZoomBox = zoomBoxExtent.Width / zoomBoxExtent.Height;
                Envelope newEnv = null;
                if (whRatioZoomBox > whRatioCurrent)
                {
                    // use width
                    double mapWidthPixels = _Map.Width;
                    double multiplier = currentExtent.Width / zoomBoxExtent.Width;
                    double newWidthMapUnits = currentExtent.Width * multiplier;
                    newEnv = new Envelope(new MapPoint(zoomBoxCenter.X - (newWidthMapUnits / 2), zoomBoxCenter.Y),
                                          new MapPoint(zoomBoxCenter.X + (newWidthMapUnits / 2), zoomBoxCenter.Y));
                }
                else
                {
                    // use height
                    double mapHeightPixels = _Map.Height;
                    double multiplier = currentExtent.Height / zoomBoxExtent.Height;
                    double newHeightMapUnits = currentExtent.Height * multiplier;
                    newEnv = new Envelope(new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y - (newHeightMapUnits / 2)),
                                          new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y + (newHeightMapUnits / 2)));
                }
                if (newEnv != null)
                    _Map.ZoomTo(newEnv);
            }
        }

        private void _Map_ExtentChanged(Object sender, ExtentEventArgs e)
        {
            if (e.OldExtent == null)
            {
                _listExtentHistory.Add(e.NewExtent.Clone());
                return;
            }
            if (_bNewExtent)
            {
                _iCurrentExtentIndex += 1;
                if (_listExtentHistory.Count - _iCurrentExtentIndex > 0)
                    _listExtentHistory.RemoveRange(_iCurrentExtentIndex, (_listExtentHistory.Count - _iCurrentExtentIndex));
                if (btnZoomNext.IsEnabled)
                    btnZoomNext.IsEnabled = false;
                _listExtentHistory.Add(e.NewExtent.Clone());
                if (!btnZoomPrev.IsEnabled)
                    btnZoomPrev.IsEnabled = true;
            }
            else
            {
                _Map.IsHitTestVisible = true;
                _bNewExtent = true;
            }
        }

        private void _Map_MouseClick(Object sender, Map.MouseEventArgs e)
        {
            if (_btnCurrentTool == null)
                return;
            else if (_btnCurrentTool == btnIdentify)
                _Parent.Identify(e.MapPoint);
        }

        private void btnZoomIn_Click(Object sender, RoutedEventArgs e)
        {
            SetTool(btnZoomIn);
        }

        private void btnZoomOut_Click(Object sender, RoutedEventArgs e)
        {
            SetTool(btnZoomOut);
        }

        private void btnPan_Click(Object sender, RoutedEventArgs e)
        {
            SetTool(btnPan);
        }

        private void btnZoomPrev_Click(Object sender, RoutedEventArgs e)
        {
            if (_iCurrentExtentIndex != 0)
            {
                _iCurrentExtentIndex -= 1;
                if (_iCurrentExtentIndex == 0)
                    btnZoomPrev.IsEnabled = false;
                _bNewExtent = false;
                _Map.IsHitTestVisible = false;
                _Map.ZoomTo(_listExtentHistory[_iCurrentExtentIndex]);
                if (!btnZoomNext.IsEnabled)
                    btnZoomNext.IsEnabled = true;
            }
        }

        private void btnZoomNext_Click(Object sender, RoutedEventArgs e)
        {
            if (_iCurrentExtentIndex < _listExtentHistory.Count - 1)
            {
                _iCurrentExtentIndex += 1;
                if (_iCurrentExtentIndex == _listExtentHistory.Count - 1)
                    btnZoomNext.IsEnabled = false;
                _bNewExtent = false;
                _Map.IsHitTestVisible = false;
                _Map.ZoomTo(_listExtentHistory[_iCurrentExtentIndex]);
                if (!btnZoomPrev.IsEnabled)
                    btnZoomPrev.IsEnabled = true;
            }
        }

        private void btnZoomFull_Click(Object sender, RoutedEventArgs e)
        {
            _Map.ZoomTo(_Map.Layers.GetFullExtent());
        }

        private void btnIdentify_Click(Object sender, RoutedEventArgs e)
        {
            SetTool(btnIdentify);
        }

        private void btnMeasure_Click(Object sender, RoutedEventArgs e)
        {
            _Parent.ShowMeasureDialog();
        }
        #endregion

        #region Private Methods
        private void SetTool(Button btn)
        {
            _Parent.UnsetTool();
            btn.Background = (Brush) Application.Current.Resources["ButtonHighlightBrush"];
            _btnCurrentTool = btn;
            if (btn == btnZoomIn)
            {
                _ZoomDrawObject.IsEnabled = true;
                _sToolMode = "zoomin";
                _Parent.SetCursor(MouseCursorType.ZoomIn);
            }
            else if (btn == btnZoomOut)
            {
                _ZoomDrawObject.IsEnabled = true;
                _sToolMode = "zoomout";
                _Parent.SetCursor(MouseCursorType.ZoomOut);
            }
            else
            {
                _ZoomDrawObject.IsEnabled = false;
                _sToolMode = "";
                if (btn == btnPan)
                    _Parent.SetCursor(MouseCursorType.Pan);
                else if (btn == btnIdentify)
                    _Parent.SetCursor(MouseCursorType.Identify);
            }
        }
        #endregion


    }
}
