﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

namespace $safeprojectname$
{
    public partial class ResultContainer : UserControl
    {

        private enum ItemType
        {
            UnsetItem = 0,
            ResultCollectionItem = 1,
            ResultItem = 2
        }

        private Map _Map;
        private string _GraphicsLayer = "";
        private Popup _ContextMenuPopup;
        private ResultContextMenu _ContextMenu;
        private IdentifyDialog _IdentifyDialog;
        private double _dMinZoomExtent = 1;
        private Dictionary<string, Graphic> _ResultItemMap = new Dictionary<string, Graphic>();

        #region Public Methods
        public ResultContainer()
        {
            InitializeComponent();
            MyDialogBase.SetParent(this);
            MyDialogBase.CloseEvent += MyDialogBase_Close;
            MyDialogBase.CollapseEvent += MyDialogBase_Collapse;
            MyDialogBase.ExpandEvent += MyDialogBase_Expand;
            _ContextMenuPopup = new Popup();
            _ContextMenu = new ResultContextMenu();
            _ContextMenu.SetParent(this);
            _ContextMenuPopup.Child = _ContextMenu;
        }

        public void SetMap(Map TheMap)
        {
            _Map = TheMap;
        }

        public void SetMapUnits(ScaleBarUnit mapUnits)
        {
            // _dMinZoomExtent defines the smallest extent allowed
            // when zooming to one or more features
            // (You can set your own preferences here)
            switch (mapUnits)
            {
                case ScaleBarUnit.DecimalDegrees:
                    _dMinZoomExtent = 0.05;
                    break;
                case ScaleBarUnit.Feet:
                    _dMinZoomExtent = 800;
                    break;
                case ScaleBarUnit.Meters:
                    _dMinZoomExtent = 250;
                    break;
                default:
                    _dMinZoomExtent = 1;
                    break;
            }
        }

        public void SetGraphicsLayer(string sLayer)
        {
            _GraphicsLayer = sLayer;
        }

        public void SetIdentifyDialog(IdentifyDialog TheDialog)
        {
            _IdentifyDialog = TheDialog;
        }

        public void AddResults(FeatureSet fsResults, string sTaskName)
        {

            TreeViewItem tvResultsItem = new TreeViewItem();

            tvResultsItem.Header = sTaskName;
            tvResultsItem.Tag = ItemType.ResultCollectionItem;

            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            GeometryType gType;
            int iCount = 0;
            TreeViewItem tvItem;
            string sID;

            gType = fsResults.GeometryType;
            foreach (Graphic gFeat in fsResults.Features)
            {
                iCount++;
                switch (gType)
                {
                    case GeometryType.Point:
                    case GeometryType.MultiPoint:
                        gFeat.Symbol = DefaultMarkerSymbol;
                        break;
                    case GeometryType.Polyline:
                        gFeat.Symbol = DefaultLineSymbol;
                        break;
                    case GeometryType.Polygon:
                    case GeometryType.Envelope:
                        gFeat.Symbol = DefaultFillSymbol;
                        break;
                }
                graphicsLayer.Graphics.Add(gFeat);
                tvItem = new TreeViewItem();
                if (fsResults.DisplayFieldName != null)
                    tvItem.Header = gFeat.Attributes[fsResults.DisplayFieldName];
                else
                    tvItem.Header = "Feature";
                sID = Guid.NewGuid().ToString();
                tvItem.Name = sID;
                tvItem.Tag = ItemType.ResultItem;
                tvItem.Selected += TreeViewItem_Selected;
                tvItem.Unselected += TreeViewItem_UnSelected;
                tvItem.MouseRightButtonDown += TreeViewItem_MouseRightButtonDown;
                tvResultsItem.Items.Add(tvItem);
                _ResultItemMap.Add(sID, gFeat);
            }
            if (iCount == 0)
            {
                MessageBox.Show("No features found");
                return;
                //tvItem = New TreeViewItem();
                //tvItem.Header = "No features found";
                //tvItem.Tag = ItemType.UnsetItem;
                //tvResultsItem.Items.Add(tvItem);
                //tvResultsItem.Tag = ItemType.UnsetItem;
            }
            tvResultsItem.Selected += TreeViewItem_Selected;
            tvResultsItem.Unselected += TreeViewItem_UnSelected;
            tvResultsItem.MouseRightButtonDown += TreeViewItem_MouseRightButtonDown;
            tvResults.Items.Add(tvResultsItem);
            if (this.Visibility == Visibility.Collapsed)
                this.Visibility = Visibility.Visible;

        }

        public void ZoomItem(TreeViewItem TheItem)
        {
            _ContextMenuPopup.IsOpen = false;
            Envelope envExtent = GetItemExtent(TheItem);
            if (envExtent == null)
                return;
            _Map.ZoomTo(envExtent);
        }

        public void PanItem(TreeViewItem TheItem)
        {
            _ContextMenuPopup.IsOpen = false;
            Envelope envExtent = GetItemExtent(TheItem);
            if (envExtent == null)
                return;
            _Map.PanTo(envExtent);
        }

        public void IdentifyItem(TreeViewItem TheItem)
        {
            _ContextMenuPopup.IsOpen = false;
            ItemType iType;
            try
            {
                iType = (ItemType)TheItem.Tag;
            }
            catch
            {
                return; // This should never happen
            }
            string sTitle, sID;
            switch (iType)
            {
                case ItemType.ResultCollectionItem:
                    bool bFirst = true;
                    foreach (TreeViewItem tvi in TheItem.Items)
                    {
                        sTitle = tvi.Header.ToString();
                        sID = tvi.Name;
                        _IdentifyDialog.IdentifyFeature(_ResultItemMap[sID], sTitle, bFirst);
                        bFirst = false;
                    }
                    break;
                case ItemType.ResultItem:
                    sTitle = TheItem.Header.ToString();
                    sID = TheItem.Name;
                    _IdentifyDialog.IdentifyFeature(_ResultItemMap[sID], sTitle, true);
                    break;
            }
        }

        public void UnselectItem(TreeViewItem TheItem)
        {
            _ContextMenuPopup.IsOpen = false;
            if (TheItem.IsSelected)
                TheItem.IsSelected = false;
            else
                ToggleHighlight(TheItem, false);
        }

        public void DeleteItem(TreeViewItem TheItem)
        {
            _ContextMenuPopup.IsOpen = false;
            ItemType iType;
            try
            {
                iType = (ItemType)TheItem.Tag;
            }
            catch
            {
                return; // This should never happen
            }
            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            string sID;
            Graphic g;
            switch (iType)
            {
                case ItemType.ResultCollectionItem:
                    foreach (TreeViewItem tvItem in TheItem.Items)
                    {
                        sID = tvItem.Name;
                        g = _ResultItemMap[sID];
                        graphicsLayer.Graphics.Remove(g);
                    }
                    tvResults.Items.Remove(TheItem);
                    break;
                case ItemType.ResultItem:
                    sID = TheItem.Name;
                    g = _ResultItemMap[sID];
                    graphicsLayer.Graphics.Remove(g);
                    foreach (TreeViewItem tvItem in tvResults.Items)
                    {
                        if (tvItem.Items.Contains(TheItem))
                        {
                            tvItem.Items.Remove(TheItem);
                            tvItem.IsSelected = false;
                            break;
                        }
                    }
                    break;
            }
        }

        public void CancelItem()
        {
            _ContextMenuPopup.IsOpen = false;
        }

        public void ClearResults()
        {
            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            graphicsLayer.ClearGraphics();
            _ResultItemMap.Clear();
            tvResults.Items.Clear();
        }
        #endregion

        #region Handlers
        private void MyDialogBase_Close(Object sender, EventArgs e)
        {
            GraphicsLayer graphicsLayer = (GraphicsLayer)_Map.Layers[_GraphicsLayer];
            graphicsLayer.ClearGraphics();
            this.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Collapse(Object sender, EventArgs e)
        {
            QueryResultsPanel.Visibility = Visibility.Collapsed;
        }

        private void MyDialogBase_Expand(Object sender, EventArgs e)
        {
            QueryResultsPanel.Visibility = Visibility.Visible;
        }

        private void TreeViewItem_Selected(Object sender, RoutedEventArgs e)
        {
            // Highlight selected result collection or result item
            TreeViewItem tvItem = (TreeViewItem)sender;
            ToggleHighlight(tvItem, true);
        }

        private void TreeViewItem_UnSelected(Object sender, RoutedEventArgs e)
        {
            // Unhighlight selected result collection or result item
            TreeViewItem tvItem = (TreeViewItem)sender;
            ToggleHighlight(tvItem, false);
        }

        private void TreeViewItem_MouseRightButtonDown(Object sender, MouseButtonEventArgs e)
        {
            // Invoke context menu
            TreeViewItem tvItem = (TreeViewItem)sender;
            _ContextMenu.SetItem(tvItem);
            Point p = e.GetPosition(null);
            _ContextMenuPopup.HorizontalOffset = p.X;
            _ContextMenuPopup.VerticalOffset = p.Y;
            _ContextMenuPopup.IsOpen = true;
            e.Handled = true; // This keeps the default Silverlight context menu from appearing!
        }
        #endregion

        #region Private Methods
        private Envelope GetItemExtent(TreeViewItem TheItem)
        {

            ItemType iType;
            Envelope envExtent, env;
            string sID;
            bool bFirst = true;

            try
            {
                iType = (ItemType)TheItem.Tag;
            }
            catch
            {
                return null; // This should never happen
            }
            envExtent = null;
            switch (iType)
            {
                case ItemType.ResultCollectionItem:
                    foreach (TreeViewItem tvItem in TheItem.Items)
                    {
                        sID = tvItem.Name;
                        env = _ResultItemMap[sID].Geometry.Extent;
                        if (bFirst)
                        {
                            envExtent = env;
                            bFirst = false;
                        }
                        else
                            envExtent = envExtent.Union(env);
                    }
                    break;
                case ItemType.ResultItem:
                    sID = TheItem.Name;
                    envExtent = _ResultItemMap[sID].Geometry.Extent;
                    break;
            }
            double d = envExtent.Width;
            if (d < _dMinZoomExtent)
            {
                d = (_dMinZoomExtent - d) / 2;
                envExtent.XMin = envExtent.XMin - d;
                envExtent.XMax = envExtent.XMax + d;
            }
            double dWidth = envExtent.Width / 20;
            double dHeight = envExtent.Height / 20;
            envExtent.XMin = envExtent.XMin - dWidth;
            envExtent.YMin = envExtent.YMin - dHeight;
            envExtent.XMax = envExtent.XMax + dWidth;
            envExtent.YMax = envExtent.YMax + dHeight;

            return envExtent;
        }

        private void ToggleHighlight(TreeViewItem TheItem, bool bHighlight)
        {

            ItemType iType;
            string sID;
            Graphic gFeat;

            try
            {
                iType = (ItemType) TheItem.Tag;
            }
            catch
            {
                return; // This should never happen
            }
            switch (iType)
            {
                case ItemType.ResultCollectionItem:
                    foreach (TreeViewItem tvItem in TheItem.Items)
                    {
                        sID = tvItem.Name;
                        gFeat = _ResultItemMap[sID];
                        ToggleGraphic(gFeat, bHighlight);
                    }
                    break;
                case ItemType.ResultItem:
                    sID = TheItem.Name;
                    gFeat = _ResultItemMap[sID];
                    ToggleGraphic(gFeat, bHighlight);
                    break;
            }
            
        }

        private void ToggleGraphic(Graphic gFeat, bool bHighlight)
        {
            ESRI.ArcGIS.Client.Geometry.Geometry geom = gFeat.Geometry;
            if ((geom is MapPoint) || (geom is MultiPoint))
            {
                if (bHighlight)
                    gFeat.Symbol = HighlightMarkerSymbol;
                else
                    gFeat.Symbol = DefaultMarkerSymbol;
            }
            else if (geom is ESRI.ArcGIS.Client.Geometry.Polyline)
            {
                if (bHighlight)
                    gFeat.Symbol = HighlightLineSymbol;
                else
                    gFeat.Symbol = DefaultLineSymbol;
            }
            else if ((geom is ESRI.ArcGIS.Client.Geometry.Polygon) || (geom is Envelope))
            {
                if (bHighlight)
                    gFeat.Symbol = HighlightFillSymbol;
                else
                    gFeat.Symbol = DefaultFillSymbol;
            }
        }
        #endregion

    }
}
