﻿Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry

Partial Public Class MainPage
    Inherits UserControl

    Private _TaskList() As String = New String() {"Locate City", _
                                                  "Locate County"}

    Public Sub New()
        InitializeComponent()
        ' Initialize mouse cursor handling
        MyMouseCursor.SetParent(Me)
        MyMouseCursor.SetMap(MyMap)
        ' Initialize navigation toolbar and associated dialogs
        MyNavToolbar.SetParent(Me)
        MyNavToolbar.SetMap(MyMap)
        MyIdentifyDialog.SetMap(MyMap)
        MyIdentifyDialog.SetGraphicsLayer("IDGraphicsLayer")
        MyIdentifyDialog.SetTask("http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Demographics/ESRI_Census_USA/MapServer")
        MyMeasureDialog.SetParent(Me)
        MyMeasureDialog.SetMap(MyMap)
        MyMeasureDialog.SetMapUnits(ScaleBarUnit.DecimalDegrees)
        MyMeasureDialog.SetGraphicsLayer("MeasureGraphicsLayer")
        ' Initialize query tasks
        MyTaskMenu.SetParent(Me)
        MyResultContainer.SetMap(MyMap)
        MyResultContainer.SetMapUnits(ScaleBarUnit.DecimalDegrees)
        MyResultContainer.SetGraphicsLayer("ResultsGraphicsLayer")
        MyResultContainer.SetIdentifyDialog(MyIdentifyDialog)
        MyCityLocator.SetResultContainer(MyResultContainer)
        MyCountyLocator.SetResultContainer(MyResultContainer)
    End Sub

    ' Mouse Cursor Handling
    Public Sub SetCursor(ByVal mcType As MouseCursorType)
        MyMouseCursor.SetCursor(mcType)
    End Sub

    ' Tool Handling
    Public Sub UnsetTool()
        MyNavToolbar.UnsetTool()
        MyMeasureDialog.UnsetTool()
        MyMouseCursor.SetCursor(MouseCursorType.None)
    End Sub

    Public Sub Identify(ByVal ptMapPoint As MapPoint)
        MyIdentifyDialog.Identify(ptMapPoint)
    End Sub

    Public Sub ShowMeasureDialog()
        MyMeasureDialog.Visibility = Visibility.Visible
    End Sub

    ' Task Menu Handling
    Public Function GetTaskList() As String()
        Return _TaskList
    End Function

    Public Sub OpenTask(ByVal sTask As String)
        Select Case sTask
            Case "Locate City"
                MyCityLocator.Visibility = Visibility.Visible
            Case "Locate County"
                MyCountyLocator.Visibility = Visibility.Visible
        End Select
    End Sub

    ' Sizing Support

    Private Sub Canvas_SizeChanged(ByVal sender As Object, ByVal e As SizeChangedEventArgs) Handles LayoutCanvas.SizeChanged
        MyMap.Height = e.NewSize.Height
        MyMap.Width = e.NewSize.Width
        Dim dY0 As Double = Canvas.GetTop(MyScaleBar)
        Dim dY As Double = e.NewSize.Height - dY0 - 30
        ScaleBarTransform.Y = dY
    End Sub


End Class