﻿Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry
Imports ESRI.ArcGIS.Client.Tasks

Partial Public Class CountyLocator
    Inherits UserControl

    Private Const _StateQueryTaskURL As String = "http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Specialty/ESRI_StatesCitiesRivers_USA/MapServer/2"
    Private Const _CountyQueryTaskURL As String = "http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Demographics/ESRI_Census_USA/MapServer/3"
    Private _ResultContainer As ResultContainer

    Public Sub New()
        InitializeComponent()
        MyDialogBase.SetParent(Me)
        AddHandler MyDialogBase.CloseEvent, AddressOf MyDialogBase_Close
        AddHandler MyDialogBase.CollapseEvent, AddressOf MyDialogBase_Collapse
        AddHandler MyDialogBase.ExpandEvent, AddressOf MyDialogBase_Expand
    End Sub

    Public Sub SetResultContainer(ByVal TheContainer As ResultContainer)

        _ResultContainer = TheContainer

        ' Set up query to populate state name combobox
        Dim qt As QueryTask = New QueryTask(_StateQueryTaskURL)
        AddHandler qt.ExecuteCompleted, AddressOf QueryTask_ExecuteCompleted
        AddHandler qt.Failed, AddressOf QueryTask_Failed
        Dim q As New Query
        q.OutFields.AddRange(New String() {"state_name"})
        q.ReturnGeometry = False
        q.Where = "1=1"
        qt.ExecuteAsync(q, "State")

    End Sub

    Private Sub MyDialogBase_Close(ByVal sender As Object, ByVal e As EventArgs)
        Me.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Collapse(ByVal sender As Object, ByVal e As EventArgs)
        LocatorPanel.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Expand(ByVal sender As Object, ByVal e As EventArgs)
        LocatorPanel.Visibility = Visibility.Visible
    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnOK.Click

        ' Build whereclause
        Dim sWhereClause, sCountyName, sStateName As String
        sCountyName = tbCounty.Text
        If sCountyName = "" Then
            MessageBox.Show("No county name entered.")
            Exit Sub
        End If
        If sCountyName.Contains("*") Or sCountyName.Contains("%") Then
            sCountyName = sCountyName.Replace("*", "%")
            sWhereClause = "name LIKE '" & sCountyName & "'"
        Else
            sWhereClause = "name = '" & sCountyName & "'"
        End If
        Dim i As Integer = cbState.SelectedIndex
        If i <> 0 Then
            sStateName = cbState.Items(i).ToString
            sWhereClause = sWhereClause & " AND state_name = '" & sStateName & "'"
        End If

        ' Execute query
        Dim qt As QueryTask = New QueryTask(_CountyQueryTaskURL)
        AddHandler qt.ExecuteCompleted, AddressOf QueryTask_ExecuteCompleted
        AddHandler qt.Failed, AddressOf QueryTask_Failed
        Dim q As New Query
        q.Where = sWhereClause
        q.ReturnGeometry = True
        q.OutFields.Add("*")
        lblStatus.Text = "Querying..."
        qt.ExecuteAsync(q, "County")

    End Sub

    Private Sub QueryTask_ExecuteCompleted(ByVal sender As Object, ByVal args As ESRI.ArcGIS.Client.Tasks.QueryEventArgs)
        Dim fs As FeatureSet = args.FeatureSet
        Dim sQueryType As String = TryCast(args.UserState, String)
        If sQueryType = "State" Then
            PopulateStates(fs)
        Else
            If fs.Features.Count = 0 Then
                MessageBox.Show("No features found")
            Else
                _ResultContainer.AddResults(fs, "Locate County")
            End If
            lblStatus.Text = "Done"
        End If
    End Sub

    Private Sub QueryTask_Failed(ByVal sender As Object, ByVal args As TaskFailedEventArgs)
        lblStatus.Text = "Error"
        MessageBox.Show("Query failed: " & args.Error.Message)
    End Sub

    Private Sub PopulateStates(ByVal fs As FeatureSet)
        Dim sl As New List(Of String)
        For Each g As Graphic In fs.Features
            sl.Add(g.Attributes("STATE_NAME").ToString)
        Next
        sl.Sort()
        cbState.Items.Clear()
        cbState.Items.Add("")
        For Each s As String In sl
            cbState.Items.Add(s)
        Next
        cbState.SelectedIndex = 0
    End Sub

End Class
