﻿' **** Implementation of dialog with close, collapse, and expand behavior ****
' Parent must be arranged on a Canvas for this to work properly, 
' otherwise any resizing behavior will conflict with the
' layout mechanism.  (It can be done, but it's more difficult.)

Partial Public Class DialogBase
    Inherits UserControl

    Private Const COLLAPSEHEIGHT As Double = 28

    Private _Parent As UserControl = Nothing
    Private _bShowTitle As Boolean = True
    Private _bShowCollapseButton As Boolean = True
    Private _bShowCloseButton As Boolean = True
    Private _bCollapsed As Boolean = False
    Private _dExpandedHeight As Double
    Private _bResizable As Boolean

    Public Delegate Sub CloseEventHandler(ByVal sender As Object, ByVal e As EventArgs)
    Public Event CloseEvent As CloseEventHandler
    Public Delegate Sub CollapseEventHandler(ByVal sender As Object, ByVal e As EventArgs)
    Public Event CollapseEvent As CollapseEventHandler
    Public Delegate Sub ExpandEventHandler(ByVal sender As Object, ByVal e As EventArgs)
    Public Event ExpandEvent As ExpandEventHandler

    Public Sub New()
        InitializeComponent()
    End Sub

    Public Sub SetParent(ByVal TheParent As UserControl)
        _Parent = TheParent
        MyFloatingPanel.SetParent(TheParent)
    End Sub

    Public Property Floating() As Boolean
        Get
            Return MyFloatingPanel.Floating
        End Get
        Set(ByVal value As Boolean)
            MyFloatingPanel.Floating = value
        End Set
    End Property

    Public Property Resizable() As Boolean
        Get
            Return MyFloatingPanel.Resizable
        End Get
        Set(ByVal value As Boolean)
            MyFloatingPanel.Resizable = value
        End Set
    End Property

    Public Property ShowDialogTitle() As Boolean
        Get
            Return _bShowTitle
        End Get
        Set(ByVal value As Boolean)
            _bShowTitle = value
            If _bShowTitle Then
                MyTitle.Visibility = Visibility.Visible
            Else
                MyTitle.Visibility = Visibility.Collapsed
            End If
        End Set
    End Property

    Public Property DialogTitleText() As String
        Get
            Return MyTitle.Text
        End Get
        Set(ByVal value As String)
            MyTitle.Text = value
        End Set
    End Property

    Public Property ShowCollapseButton() As Boolean
        Get
            Return _bShowCollapseButton
        End Get
        Set(ByVal value As Boolean)
            _bShowCollapseButton = value
            If _bShowCollapseButton Then
                MyCollapseButton.Visibility = Visibility.Visible
            Else
                MyCollapseButton.Visibility = Visibility.Collapsed
            End If
        End Set
    End Property

    Public Property ShowCloseButton() As Boolean
        Get
            Return _bShowCloseButton
        End Get
        Set(ByVal value As Boolean)
            _bShowCloseButton = value
            If _bShowCloseButton Then
                MyCloseButton.Visibility = Visibility.Visible
            Else
                MyCloseButton.Visibility = Visibility.Collapsed
            End If
        End Set
    End Property

    Private Sub MyCollapseButton_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles MyCollapseButton.Click
        Dim ucTarget As UserControl
        If _Parent Is Nothing Then
            ucTarget = Me
        Else
            ucTarget = _Parent
        End If
        Dim args As EventArgs = EventArgs.Empty
        Dim img As Image = MyCollapseButton.Content
        If _bCollapsed Then
            ' Expand dialog
            If _bResizable Then
                MyFloatingPanel.Resizable = True
            End If
            ucTarget.Height = _dExpandedHeight
            If Not _Parent Is Nothing Then
                RaiseEvent ExpandEvent(Me, args)
            End If
            img.Style = Application.Current.Resources("CollapseButtonImage")
            _bCollapsed = False
        Else
            ' Collapse dialog
            _bResizable = MyFloatingPanel.Resizable
            If _bResizable Then
                MyFloatingPanel.Resizable = False
            End If
            _dExpandedHeight = ucTarget.ActualHeight
            If Not _Parent Is Nothing Then
                RaiseEvent CollapseEvent(Me, args)
            End If
            ucTarget.Height = COLLAPSEHEIGHT
            img.Style = Application.Current.Resources("ExpandButtonImage")
            _bCollapsed = True
        End If

    End Sub

    Private Sub MyCloseButton_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles MyCloseButton.Click
        If _Parent Is Nothing Then
            Me.Visibility = Visibility.Collapsed
        Else
            Dim args As EventArgs = EventArgs.Empty
            RaiseEvent CloseEvent(Me, args)
        End If
    End Sub

End Class
