﻿' **** Implementation of floating, resizable placeholder ****
' Parent must be arranged on a Canvas for this to work properly, 
' otherwise any resizing behavior will conflict with the
' layout mechanism.  (It can be done, but it's more difficult.)

Partial Public Class FloatingPanel
    Inherits UserControl

    Private _Parent As UserControl = Nothing
    Private _bFloating As Boolean = True
    Private _bResizable As Boolean = False

    Public Sub New()
        InitializeComponent()
    End Sub

    Public Sub SetParent(ByVal TheParent As UserControl)
        _Parent = TheParent
        _Parent.RenderTransform = FloatingPanelTransform
        Me.RenderTransform = Nothing
    End Sub

    Public Property Floating() As Boolean
        Get
            Return _bFloating
        End Get
        Set(ByVal value As Boolean)
            _bFloating = value
        End Set
    End Property

    Public Property Resizable() As Boolean
        Get
            Return _bResizable
        End Get
        Set(ByVal value As Boolean)
            _bResizable = value
            If _bResizable Then
                Grip.Visibility = Visibility.Visible
                PanelThumb.Visibility = Visibility.Visible
            Else
                Grip.Visibility = Visibility.Collapsed
                PanelThumb.Visibility = Visibility.Collapsed
            End If
        End Set
    End Property

    '**** Floating behavior

    Private Sub PanelBase_DragDelta(ByVal sender As System.Object, ByVal e As System.Windows.Controls.Primitives.DragDeltaEventArgs) Handles PanelBase.DragDelta
        If Not _bFloating Then
            Exit Sub
        End If
        Dim dX As Double = e.HorizontalChange
        Dim dY As Double = e.VerticalChange
        FloatingPanelTransform.X += dX
        FloatingPanelTransform.Y += dY
    End Sub

    '**** Resizing behavior

    Private Sub PanelThumb_DragDelta(ByVal sender As System.Object, ByVal e As System.Windows.Controls.Primitives.DragDeltaEventArgs) Handles PanelThumb.DragDelta
        If Not _bResizable Then
            Exit Sub
        End If
        Dim ucTarget As UserControl
        If _Parent Is Nothing Then
            ucTarget = Me
        Else
            ucTarget = _Parent
        End If
        Dim dX As Double = ucTarget.Width + e.HorizontalChange
        Dim dY As Double = ucTarget.Height + e.VerticalChange
        If dX > ucTarget.MinWidth And dY > ucTarget.MinHeight Then
            ucTarget.Width = dX
            ucTarget.Height = dY
        End If
    End Sub

End Class
