﻿Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry
Imports ESRI.ArcGIS.Client.Tasks

'**** Adapted from ESRI's identify sample ****

Partial Public Class IdentifyDialog
    Inherits UserControl

    Private Class DataItem
        Private privateTitle As String
        Public Property Title() As String
            Get
                Return privateTitle
            End Get
            Set(ByVal value As String)
                privateTitle = value
            End Set
        End Property
        Private privateData As IDictionary(Of String, Object)
        Public Property Data() As IDictionary(Of String, Object)
            Get
                Return privateData
            End Get
            Set(ByVal value As IDictionary(Of String, Object))
                privateData = value
            End Set
        End Property
    End Class

    Private _Map As Map
    Private _GraphicsLayer As String = ""
    Private _URL As String = ""
    Private _IdentifyTask As IdentifyTask
    Private _dataItems As List(Of DataItem) = Nothing

#Region "Public Methods"
    Public Sub New()
        InitializeComponent()
        MyDialogBase.SetParent(Me)
        AddHandler MyDialogBase.CloseEvent, AddressOf MyDialogBase_Close
        AddHandler MyDialogBase.CollapseEvent, AddressOf MyDialogBase_Collapse
        AddHandler MyDialogBase.ExpandEvent, AddressOf MyDialogBase_Expand
    End Sub

    Public Sub SetMap(ByVal TheMap As Map)
        _Map = TheMap
    End Sub

    Public Sub SetGraphicsLayer(ByVal sLayer As String)
        _GraphicsLayer = sLayer
    End Sub

    Public Sub SetTask(ByVal TheURL As String)
        _URL = TheURL
        _IdentifyTask = New IdentifyTask(_URL)
        AddHandler _IdentifyTask.ExecuteCompleted, AddressOf IdentifyTask_ExecuteCompleted
        AddHandler _IdentifyTask.Failed, AddressOf IdentifyTask_Failed
    End Sub

    Public Sub Identify(ByVal ptMapPoint As MapPoint)
        If _URL = "" Then
            Exit Sub
        End If
        Dim identifyParams As ESRI.ArcGIS.Client.Tasks.IdentifyParameters = New IdentifyParameters() _
            With {.Geometry = ptMapPoint, .MapExtent = _Map.Extent, _
                  .Width = CInt(Fix(_Map.ActualWidth)), .Height = CInt(Fix(_Map.ActualHeight)), _
                  .LayerOption = LayerOption.visible}
        _IdentifyTask.ExecuteAsync(identifyParams)
        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        graphicsLayer.ClearGraphics()
        Dim graphic As New ESRI.ArcGIS.Client.Graphic() With {.Geometry = ptMapPoint, .Symbol = DefaultPictureSymbol}
        graphicsLayer.Graphics.Add(graphic)
    End Sub

    Public Sub IdentifyFeature(ByVal gFeat As Graphic, Optional ByVal sTitle As String = "Feature", Optional ByVal bClear As Boolean = True)
        If bClear Then
            IdentifyComboBox.Items.Clear()
            _dataItems = New List(Of DataItem)()
        End If
        _dataItems.Add(New DataItem() With {.Title = sTitle, .Data = gFeat.Attributes})
        If Me.Visibility = Windows.Visibility.Collapsed Then
            Me.Visibility = Windows.Visibility.Visible
        End If
        IdentifyComboBox.Items.Add(sTitle)
        IdentifyComboBox.UpdateLayout()
        IdentifyComboBox.SelectedIndex = IdentifyComboBox.Items.Count - 1
    End Sub

#End Region

#Region "Handlers"
    Private Sub MyDialogBase_Close(ByVal sender As Object, ByVal e As EventArgs)
        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        graphicsLayer.ClearGraphics()
        Me.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Collapse(ByVal sender As Object, ByVal e As EventArgs)
        IdentifyResultsPanel.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Expand(ByVal sender As Object, ByVal e As EventArgs)
        IdentifyResultsPanel.Visibility = Visibility.Visible
    End Sub

    Private Sub cb_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs) Handles IdentifyComboBox.SelectionChanged
        Dim index As Integer = IdentifyComboBox.SelectedIndex
        If index > -1 Then
            IdentifyDetailsDataGrid.ItemsSource = _dataItems(index).Data
        End If
    End Sub

    Private Sub IdentifyTask_ExecuteCompleted(ByVal sender As Object, ByVal args As IdentifyEventArgs)
        IdentifyDetailsDataGrid.ItemsSource = Nothing
        If Me.Visibility = Windows.Visibility.Collapsed Then
            Me.Visibility = Windows.Visibility.Visible
        End If
        If args.IdentifyResults IsNot Nothing AndAlso args.IdentifyResults.Count > 0 Then
            IdentifyResultsPanel.Visibility = Visibility.Visible
            ShowFeatures(args.IdentifyResults)
        Else
            IdentifyComboBox.Items.Clear()
            IdentifyComboBox.UpdateLayout()
            IdentifyResultsPanel.Visibility = Visibility.Collapsed
        End If
    End Sub

    Private Sub IdentifyTask_Failed(ByVal sender As Object, ByVal e As TaskFailedEventArgs)
        MessageBox.Show("Identify failed. Error: " & e.Error.Message)
    End Sub
#End Region

#Region "Private Methods"
    Private Sub ShowFeatures(ByVal results As List(Of IdentifyResult))
        _dataItems = New List(Of DataItem)()
        If results IsNot Nothing AndAlso results.Count > 0 Then
            IdentifyComboBox.Items.Clear()
            For Each result As IdentifyResult In results
                Dim feature As Graphic = result.Feature
                Dim title As String = result.Value.ToString() & " (" & result.LayerName & ")"
                _dataItems.Add(New DataItem() With {.Title = title, .Data = feature.Attributes})
                IdentifyComboBox.Items.Add(title)
            Next result
            IdentifyComboBox.UpdateLayout()
            IdentifyComboBox.SelectedIndex = 0
        End If
    End Sub
#End Region

End Class
