﻿Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry

'**** Adapted from ESRI's toolbar sample ****

Partial Public Class NavToolbar
    Inherits UserControl

    Private _Parent As MainPage
    Private _Map As Map
    Private _ZoomDrawObject As Draw

    Private _sToolMode As String = ""
    Private _listExtentHistory As New List(Of Envelope)()
    Private _iCurrentExtentIndex As Integer = 0
    Private _bNewExtent As Boolean = True
    Private _btnCurrentTool As Button = Nothing

    Public Sub New()
        InitializeComponent()
        MyFloatingPanel.SetParent(Me)
    End Sub

#Region "Public Methods"
    Public Sub SetParent(ByVal TheParent As MainPage)
        ' Assumes that MainPage supports SetCursor and UnsetTool
        _Parent = TheParent
    End Sub

    Public Sub SetMap(ByVal TheMap As Map)
        _Map = TheMap
        AddHandler _Map.ExtentChanged, AddressOf _Map_ExtentChanged
        AddHandler _Map.MouseClick, AddressOf _Map_MouseClick
        _ZoomDrawObject = New Draw(_Map) With {.FillSymbol = DefaultFillSymbol, .DrawMode = DrawMode.Rectangle}
        AddHandler _ZoomDrawObject.DrawComplete, AddressOf _ZoomDrawObject_DrawComplete
    End Sub

    Public Sub UnsetTool()
        If _btnCurrentTool Is Nothing Then
            Exit Sub
        End If
        _btnCurrentTool.Background = Application.Current.Resources("ButtonNormalBrush")
        _ZoomDrawObject.IsEnabled = False
        _btnCurrentTool = Nothing
    End Sub
#End Region

#Region "Handlers"
    Private Sub _ZoomDrawObject_DrawComplete(ByVal sender As Object, ByVal args As DrawEventArgs)
        If _sToolMode = "zoomin" Then
            _Map.ZoomTo(TryCast(args.Geometry, ESRI.ArcGIS.Client.Geometry.Envelope))
        ElseIf _sToolMode = "zoomout" Then
            Dim currentExtent As Envelope = _Map.Extent
            Dim zoomBoxExtent As Envelope = TryCast(args.Geometry, Envelope)
            Dim zoomBoxCenter As MapPoint = zoomBoxExtent.GetCenter()
            Dim whRatioCurrent As Double = currentExtent.Width / currentExtent.Height
            Dim whRatioZoomBox As Double = zoomBoxExtent.Width / zoomBoxExtent.Height
            Dim newEnv As Envelope = Nothing
            If whRatioZoomBox > whRatioCurrent Then
                ' use width
                Dim mapWidthPixels As Double = _Map.Width
                Dim multiplier As Double = currentExtent.Width / zoomBoxExtent.Width
                Dim newWidthMapUnits As Double = currentExtent.Width * multiplier
                newEnv = New Envelope(New MapPoint(zoomBoxCenter.X - (newWidthMapUnits / 2), zoomBoxCenter.Y), _
                                      New MapPoint(zoomBoxCenter.X + (newWidthMapUnits / 2), zoomBoxCenter.Y))
            Else
                ' use height
                Dim mapHeightPixels As Double = _Map.Height
                Dim multiplier As Double = currentExtent.Height / zoomBoxExtent.Height
                Dim newHeightMapUnits As Double = currentExtent.Height * multiplier
                newEnv = New Envelope(New MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y - (newHeightMapUnits / 2)), _
                                      New MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y + (newHeightMapUnits / 2)))
            End If
            If newEnv IsNot Nothing Then
                _Map.ZoomTo(newEnv)
            End If
        End If
    End Sub

    Private Sub _Map_ExtentChanged(ByVal sender As Object, ByVal e As ExtentEventArgs)
        If e.OldExtent Is Nothing Then
            _listExtentHistory.Add(e.NewExtent.Clone())
            Return
        End If
        If _bNewExtent Then
            _iCurrentExtentIndex += 1
            If _listExtentHistory.Count - _iCurrentExtentIndex > 0 Then
                _listExtentHistory.RemoveRange(_iCurrentExtentIndex, (_listExtentHistory.Count - _iCurrentExtentIndex))
            End If
            If btnZoomNext.IsEnabled Then
                btnZoomNext.IsEnabled = False
            End If
            _listExtentHistory.Add(e.NewExtent.Clone())
            If Not btnZoomPrev.IsEnabled Then
                btnZoomPrev.IsEnabled = True
            End If
        Else
            _Map.IsHitTestVisible = True
            _bNewExtent = True
        End If
    End Sub

    Private Sub _Map_MouseClick(ByVal sender As Object, ByVal e As ESRI.ArcGIS.Client.Map.MouseEventArgs)
        If _btnCurrentTool Is Nothing Then
            Exit Sub
        ElseIf _btnCurrentTool Is btnIdentify Then
            _Parent.Identify(e.MapPoint)
        End If
    End Sub

    Private Sub btnZoomIn_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnZoomIn.Click
        SetTool(btnZoomIn)
    End Sub

    Private Sub btnZoomOut_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnZoomOut.Click
        SetTool(btnZoomOut)
    End Sub

    Private Sub btnPan_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnPan.Click
        SetTool(btnPan)
    End Sub

    Private Sub btnZoomPrev_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnZoomPrev.Click
        If _iCurrentExtentIndex <> 0 Then
            _iCurrentExtentIndex -= 1
            If _iCurrentExtentIndex = 0 Then
                btnZoomPrev.IsEnabled = False
            End If
            _bNewExtent = False
            _Map.IsHitTestVisible = False
            _Map.ZoomTo(_listExtentHistory(_iCurrentExtentIndex))
            If Not btnZoomNext.IsEnabled Then
                btnZoomNext.IsEnabled = True
            End If
        End If
    End Sub

    Private Sub btnZoomNext_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnZoomNext.Click
        If _iCurrentExtentIndex < _listExtentHistory.Count - 1 Then
            _iCurrentExtentIndex += 1
            If _iCurrentExtentIndex = (_listExtentHistory.Count - 1) Then
                btnZoomNext.IsEnabled = False
            End If
            _bNewExtent = False
            _Map.IsHitTestVisible = False
            _Map.ZoomTo(_listExtentHistory(_iCurrentExtentIndex))
            If Not btnZoomPrev.IsEnabled Then
                btnZoomPrev.IsEnabled = True
            End If
        End If
    End Sub

    Private Sub btnZoomFull_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnZoomFull.Click
        _Map.ZoomTo(_Map.Layers.GetFullExtent())
    End Sub

    Private Sub btnIdentify_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnIdentify.Click
        SetTool(btnIdentify)
    End Sub

    Private Sub btnMeasure_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnMeasure.Click
        _Parent.ShowMeasureDialog()
    End Sub
#End Region

#Region "Private Methods"
    Private Sub SetTool(ByVal btn As Button)
        _Parent.UnsetTool()
        btn.Background = Application.Current.Resources("ButtonHighlightBrush")
        _btnCurrentTool = btn
        If btn Is btnZoomIn Then
            _ZoomDrawObject.IsEnabled = True
            _sToolMode = "zoomin"
            _Parent.SetCursor(MouseCursorType.ZoomIn)
        ElseIf btn Is btnZoomOut Then
            _ZoomDrawObject.IsEnabled = True
            _sToolMode = "zoomout"
            _Parent.SetCursor(MouseCursorType.ZoomOut)
        Else
            _ZoomDrawObject.IsEnabled = False
            _sToolMode = ""
            If btn Is btnPan Then
                _Parent.SetCursor(MouseCursorType.Pan)
            ElseIf btn Is btnIdentify Then
                _Parent.SetCursor(MouseCursorType.Identify)
            End If
        End If
    End Sub
#End Region

End Class
