﻿Imports System.Windows.Controls.Primitives
Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry
Imports ESRI.ArcGIS.Client.Tasks

Partial Public Class ResultContainer
    Inherits UserControl

    Private Enum ItemType
        UnsetItem = 0
        ResultCollectionItem = 1
        ResultItem = 2
    End Enum

    Private _Map As Map
    Private _GraphicsLayer As String = ""
    Dim _ContextMenuPopup As Popup
    Dim _ContextMenu As ResultContextMenu
    Private _IdentifyDialog As IdentifyDialog
    Private _dMinZoomExtent As Double = 1
    Private _ResultItemMap As New Dictionary(Of String, Graphic)

#Region "Public methods"
    Public Sub New()
        InitializeComponent()
        MyDialogBase.SetParent(Me)
        AddHandler MyDialogBase.CloseEvent, AddressOf MyDialogBase_Close
        _ContextMenuPopup = New Popup()
        _ContextMenu = New ResultContextMenu()
        _ContextMenu.SetParent(Me)
        _ContextMenuPopup.Child = _ContextMenu
    End Sub

    Public Sub SetMap(ByVal TheMap As Map)
        _Map = TheMap
    End Sub

    Public Sub SetMapUnits(ByVal mapUnits As ScaleBarUnit)
        ' _dMinZoomExtent defines the smallest extent allowed
        ' when zooming to one or more features
        ' (You can set your own preferences here)
        Select Case mapUnits
            Case ScaleBarUnit.DecimalDegrees
                _dMinZoomExtent = 0.05
            Case ScaleBarUnit.Feet
                _dMinZoomExtent = 800
            Case ScaleBarUnit.Meters
                _dMinZoomExtent = 250
            Case Else
                _dMinZoomExtent = 1
        End Select
    End Sub

    Public Sub SetGraphicsLayer(ByVal sLayer As String)
        _GraphicsLayer = sLayer
    End Sub

    Public Sub SetIdentifyDialog(ByVal TheDialog As IdentifyDialog)
        _IdentifyDialog = TheDialog
    End Sub

    Public Sub AddResults(ByVal fsResults As FeatureSet, ByVal sTaskName As String)

        Dim tvResultsItem As New TreeViewItem

        tvResultsItem.Header = sTaskName
        tvResultsItem.Tag = ItemType.ResultCollectionItem

        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        Dim gType As GeometryType
        Dim iCount As Integer = 0
        Dim gFeat As Graphic
        Dim tvItem As TreeViewItem
        Dim sID As String

        gType = fsResults.GeometryType
        For Each gFeat In fsResults.Features
            iCount += 1
            Select Case gType
                Case GeometryType.Point, GeometryType.MultiPoint
                    gFeat.Symbol = DefaultMarkerSymbol
                Case GeometryType.Polyline
                    gFeat.Symbol = DefaultLineSymbol
                Case GeometryType.Polygon, GeometryType.Envelope
                    gFeat.Symbol = DefaultFillSymbol
            End Select
            graphicsLayer.Graphics.Add(gFeat)
            tvItem = New TreeViewItem
            If Not fsResults.DisplayFieldName Is Nothing Then
                tvItem.Header = gFeat.Attributes(fsResults.DisplayFieldName)
            Else
                tvItem.Header = "Feature"
            End If
            sID = Guid.NewGuid.ToString
            tvItem.Name = sID
            tvItem.Tag = ItemType.ResultItem
            AddHandler tvItem.Selected, AddressOf TreeViewItem_Selected
            AddHandler tvItem.Unselected, AddressOf TreeViewItem_UnSelected
            AddHandler tvItem.MouseRightButtonDown, AddressOf TreeViewItem_MouseRightButtonDown
            tvResultsItem.Items.Add(tvItem)
            _ResultItemMap.Add(sID, gFeat)
        Next
        If iCount = 0 Then
            MessageBox.Show("No features found")
            Exit Sub
            'tvItem = New TreeViewItem
            'tvItem.Header = "No features found"
            'tvItem.Tag = ItemType.UnsetItem
            'tvResultsItem.Items.Add(tvItem)
            'tvResultsItem.Tag = ItemType.UnsetItem
        End If
        AddHandler tvResultsItem.Selected, AddressOf TreeViewItem_Selected
        AddHandler tvResultsItem.Unselected, AddressOf TreeViewItem_UnSelected
        AddHandler tvResultsItem.MouseRightButtonDown, AddressOf TreeViewItem_MouseRightButtonDown
        tvResults.Items.Add(tvResultsItem)
        If Me.Visibility = Visibility.Collapsed Then
            Me.Visibility = Visibility.Visible
        End If

    End Sub

    Public Sub ZoomItem(ByVal TheItem As TreeViewItem)
        _ContextMenuPopup.IsOpen = False
        Dim envExtent As Envelope = GetItemExtent(TheItem)
        If envExtent Is Nothing Then
            Exit Sub
        End If
        _Map.ZoomTo(envExtent)
    End Sub

    Public Sub PanItem(ByVal TheItem As TreeViewItem)
        _ContextMenuPopup.IsOpen = False
        Dim envExtent As Envelope = GetItemExtent(TheItem)
        If envExtent Is Nothing Then
            Exit Sub
        End If
        _Map.PanTo(envExtent)
    End Sub

    Public Sub IdentifyItem(ByVal TheItem As TreeViewItem)
        _ContextMenuPopup.IsOpen = False
        Dim iType As ItemType
        Try
            iType = TheItem.Tag
        Catch ex As Exception
            Exit Sub ' This should never happen
        End Try
        Dim sTitle, sID As String
        Select Case iType
            Case ItemType.ResultCollectionItem
                Dim bFirst As Boolean = True
                For Each tvi As TreeViewItem In TheItem.Items
                    sTitle = tvi.Header
                    sID = tvi.Name
                    _IdentifyDialog.IdentifyFeature(_ResultItemMap(sID), sTitle, bFirst)
                    bFirst = False
                Next
            Case ItemType.ResultItem
                sTitle = TheItem.Header
                sID = TheItem.Name
                _IdentifyDialog.IdentifyFeature(_ResultItemMap(sID), sTitle, True)
        End Select
    End Sub

    Public Sub UnselectItem(ByVal TheItem As TreeViewItem)
        _ContextMenuPopup.IsOpen = False
        If TheItem.IsSelected Then
            TheItem.IsSelected = False
        Else
            ToggleHighlight(TheItem, False)
        End If
    End Sub

    Public Sub DeleteItem(ByVal TheItem As TreeViewItem)
        _ContextMenuPopup.IsOpen = False
        Dim iType As ItemType
        Try
            iType = TheItem.Tag
        Catch ex As Exception
            Exit Sub ' This should never happen
        End Try
        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        Dim tvItem As TreeViewItem
        Dim sID As String
        Dim g As Graphic
        Select Case iType
            Case ItemType.ResultCollectionItem
                For Each tvItem In TheItem.Items
                    sID = tvItem.Name
                    g = _ResultItemMap(sID)
                    graphicsLayer.Graphics.Remove(g)
                Next
                tvResults.Items.Remove(TheItem)
            Case ItemType.ResultItem
                sID = TheItem.Name
                g = _ResultItemMap(sID)
                graphicsLayer.Graphics.Remove(g)
                For Each tvItem In tvResults.Items
                    If tvItem.Items.Contains(TheItem) Then
                        tvItem.Items.Remove(TheItem)
                        tvItem.IsSelected = False
                        Exit For
                    End If
                Next
        End Select
    End Sub

    Public Sub CancelItem()
        _ContextMenuPopup.IsOpen = False
    End Sub

    Public Sub ClearResults()
        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        graphicsLayer.ClearGraphics()
        _ResultItemMap.Clear()
        tvResults.Items.Clear()
    End Sub
#End Region

#Region "Handlers"
    Private Sub MyDialogBase_Close(ByVal sender As Object, ByVal e As EventArgs)
        Dim graphicsLayer As GraphicsLayer = TryCast(_Map.Layers(_GraphicsLayer), GraphicsLayer)
        graphicsLayer.ClearGraphics()
        Me.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Collapse(ByVal sender As Object, ByVal e As EventArgs)
        QueryResultsPanel.Visibility = Visibility.Collapsed
    End Sub

    Private Sub MyDialogBase_Expand(ByVal sender As Object, ByVal e As EventArgs)
        QueryResultsPanel.Visibility = Visibility.Visible
    End Sub

    Private Sub TreeViewItem_Selected(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Highlight selected result collection or result item
        Dim tvItem As TreeViewItem = TryCast(sender, TreeViewItem)
        ToggleHighlight(tvItem, True)
    End Sub

    Private Sub TreeViewItem_UnSelected(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Unhighlight selected result collection or result item
        Dim tvItem As TreeViewItem = TryCast(sender, TreeViewItem)
        ToggleHighlight(tvItem, False)
    End Sub

    Private Sub TreeViewItem_MouseRightButtonDown(ByVal sender As System.Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
        ' Invoke context menu
        Dim tvItem As TreeViewItem = TryCast(sender, TreeViewItem)
        _ContextMenu.SetItem(tvItem)
        Dim p As Point = e.GetPosition(Nothing)
        _ContextMenuPopup.HorizontalOffset = p.X
        _ContextMenuPopup.VerticalOffset = p.Y
        _ContextMenuPopup.IsOpen = True
        e.Handled = True ' This keeps the default Silverlight context menu from appearing!
    End Sub
#End Region

#Region "Private methods"
    Private Function GetItemExtent(ByVal TheItem As TreeViewItem) As Envelope

        Dim iType As ItemType
        Dim envExtent, env As Envelope
        Dim sID As String
        Dim bFirst As Boolean = True

        Try
            iType = TheItem.Tag
        Catch ex As Exception
            Return Nothing
        End Try
        envExtent = Nothing
        Select Case iType
            Case ItemType.ResultCollectionItem
                For Each tvItem As TreeViewItem In TheItem.Items
                    sID = tvItem.Name
                    env = _ResultItemMap(sID).Geometry.Extent
                    If bFirst Then
                        envExtent = env
                        bFirst = False
                    Else
                        envExtent = envExtent.Union(env)
                    End If
                Next
            Case ItemType.ResultItem
                sID = TheItem.Name
                envExtent = _ResultItemMap(sID).Geometry.Extent
        End Select
        Dim d As Double = envExtent.Width
        If d < _dMinZoomExtent Then
            d = (_dMinZoomExtent - d) / 2
            envExtent.XMin = envExtent.XMin - d
            envExtent.XMax = envExtent.XMax + d
        End If
        Dim dWidth As Double = envExtent.Width / 20
        Dim dHeight As Double = envExtent.Height / 20
        envExtent.XMin = envExtent.XMin - dWidth
        envExtent.YMin = envExtent.YMin - dHeight
        envExtent.XMax = envExtent.XMax + dWidth
        envExtent.YMax = envExtent.YMax + dHeight

        Return envExtent

    End Function

    Private Sub ToggleHighlight(ByVal TheItem As TreeViewItem, ByVal bHighlight As Boolean)

        Dim iType As ItemType
        Dim sID As String
        Dim gFeat As Graphic

        Try
            iType = TheItem.Tag
        Catch ex As Exception
            Exit Sub ' This should never happen
        End Try
        Select Case iType
            Case ItemType.ResultCollectionItem
                For Each tvItem As TreeViewItem In TheItem.Items
                    sID = tvItem.Name
                    gFeat = _ResultItemMap(sID)
                    ToggleGraphic(gFeat, bHighlight)
                Next
            Case ItemType.ResultItem
                sID = TheItem.Name
                gFeat = _ResultItemMap(sID)
                ToggleGraphic(gFeat, bHighlight)
        End Select

    End Sub

    Private Sub ToggleGraphic(ByVal gFeat As Graphic, ByVal bHighlight As Boolean)
        Dim geom As Geometry = gFeat.Geometry
        If TypeOf geom Is MapPoint Or TypeOf geom Is MultiPoint Then
            If bHighlight Then
                gFeat.Symbol = HighlightMarkerSymbol
            Else
                gFeat.Symbol = DefaultMarkerSymbol
            End If
        ElseIf TypeOf geom Is Polyline Then
            If bHighlight Then
                gFeat.Symbol = HighlightLineSymbol
            Else
                gFeat.Symbol = DefaultLineSymbol
            End If
        ElseIf TypeOf geom Is Polygon Or TypeOf geom Is Envelope Then
            If bHighlight Then
                gFeat.Symbol = HighlightFillSymbol
            Else
                gFeat.Symbol = DefaultFillSymbol
            End If
        End If
    End Sub
#End Region


End Class
